package backend

import (
	"database/sql"
	"errors"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/auth"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/movies"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/shows"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/torrents"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/users"

	"github.com/Sirupsen/logrus"
	"github.com/jmoiron/sqlx"
	polochon "github.com/odwrtw/polochon/lib"
)

// Backend represents the data backend
type Backend struct {
	Database *sqlx.DB
}

// GetUser gets the username from the UserBackend
// Implements the UserBackend interface
func (b *Backend) GetUser(username string) (auth.User, error) {
	return users.Get(b.Database, username)
}

// Name implements the Module interface
func (b *Backend) Name() string {
	return "canape-backend"
}

// GetDetails implements the polochon Detailer interface
func (b *Backend) GetDetails(media interface{}, log *logrus.Entry) error {
	switch t := media.(type) {
	case *polochon.Show:
		return b.GetShowDetails(t, log)
	case *polochon.ShowEpisode:
		return b.GetShowEpisodeDetails(t, log)
	case *polochon.Movie:
		return b.GetMovieDetails(t, log)
	default:
		return errors.New("invalid type")
	}
}

// GetMovieDetails gets details for movies
func (b *Backend) GetMovieDetails(pmovie *polochon.Movie, log *logrus.Entry) error {
	m := movies.New(pmovie.ImdbID)
	if err := m.GetMovie(b.Database); err != nil {
		return err
	}
	log.Debugf("got movie %s from backend", pmovie.ImdbID)
	*pmovie = m.Movie
	return nil
}

// GetShowDetails gets details for shows
func (b *Backend) GetShowDetails(pshow *polochon.Show, log *logrus.Entry) error {
	s := shows.New(pshow.ImdbID)
	if err := s.GetShow(b.Database); err != nil {
		return err
	}
	log.Debugf("got show %s from backend", pshow.ImdbID)
	*pshow = s.Show
	return nil
}

// GetShowEpisodeDetails gets details for episodes
func (b *Backend) GetShowEpisodeDetails(pepisode *polochon.ShowEpisode, log *logrus.Entry) error {
	e := shows.NewEpisode()
	e.ShowImdbID = pepisode.ShowImdbID
	e.Season = pepisode.Season
	e.Episode = pepisode.Episode
	if err := e.GetEpisode(b.Database); err != nil {
		return err
	}
	log.Debugf("got episode S%02dE%02d %s from backend", pepisode.Season, pepisode.Episode, pepisode.ShowImdbID)
	*pepisode = e.ShowEpisode
	return nil
}

// GetTorrents implements the polochon Torrenter interface
func (b *Backend) GetTorrents(media interface{}, log *logrus.Entry) error {
	switch t := media.(type) {
	case *polochon.ShowEpisode:
		return b.GetEpisodeTorrents(t, log)
	case *polochon.Movie:
		return b.GetMovieTorrents(t, log)
	default:
		return errors.New("invalid type")
	}
}

// GetMovieTorrents fetch Torrents for movies
func (b *Backend) GetMovieTorrents(pmovie *polochon.Movie, log *logrus.Entry) error {
	// m := movies.New(pmovie.ImdbID)
	movieTorrents, err := torrents.GetMovieTorrents(b.Database, pmovie.ImdbID)
	switch err {
	case nil:
		log.Debug("torrents found in backend")
	case sql.ErrNoRows:
		log.Debug("torrent not found in backend")
	default:
		// Unexpected error
		return err
	}
	log.Debugf("got torrents for movie %s from backend", pmovie.ImdbID)

	for _, t := range movieTorrents {
		pmovie.Torrents = append(pmovie.Torrents, t.Torrent)
	}

	return nil
}

// GetEpisodeTorrents fetch Torrents for episodes
func (b *Backend) GetEpisodeTorrents(pepisode *polochon.ShowEpisode, log *logrus.Entry) error {
	e := shows.NewEpisode()
	e.ShowImdbID = pepisode.ShowImdbID
	e.Season = pepisode.Season
	e.Episode = pepisode.Episode
	episodeTorrents, err := torrents.GetEpisodeTorrents(
		b.Database,
		pepisode.ShowImdbID,
		pepisode.Season,
		pepisode.Episode,
	)
	switch err {
	case nil:
		log.Debug("torrents found in backend")
	case sql.ErrNoRows:
		log.Debug("torrent not found in backend")
	default:
		// Unexpected error
		return err
	}
	log.Debugf("got torrents for episode S%02dE%02d %s from backend", pepisode.Season, pepisode.Episode, pepisode.ShowImdbID)
	// Add the torrents to the episode
	for _, t := range episodeTorrents {
		e.Torrents = append(e.Torrents, t.Torrent)
	}

	return nil
}
