package main

import (
	"net/http"
	"os"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/auth"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/backend"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/config"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/external_medias"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/movies"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/shows"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/torrents"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/users"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/web"

	"github.com/Sirupsen/logrus"
	"github.com/jmoiron/sqlx"
	_ "github.com/lib/pq"
	"github.com/phyber/negroni-gzip/gzip"
	"github.com/urfave/negroni"
)

func main() {
	var cfgPath string
	cfgPath = os.Getenv("CONFIG_FILE")
	if cfgPath == "" {
		cfgPath = "./config.yml"
	}

	// Setup the logger
	logger := logrus.New()
	logger.Formatter = &logrus.TextFormatter{FullTimestamp: true}
	logger.Level = logrus.DebugLevel

	log := logrus.NewEntry(logger)
	cf, err := config.Load(cfgPath)
	if err != nil {
		log.Panic(err)
	}

	// Connect to the database
	db, err := sqlx.Connect("postgres", cf.PGDSN)
	if err != nil {
		log.Panic(err)
	}
	backend := &backend.Backend{Database: db}

	// Generate auth params
	authParams := auth.Params{
		Backend: backend,
		Pepper:  cf.Authorizer.Pepper,
		Cost:    cf.Authorizer.Cost,
		Secret:  cf.Authorizer.Secret,
	}
	authorizer := auth.New(authParams)

	// Create web environment needed by the app
	env := web.NewEnv(web.EnvParams{
		Database: db,
		Auth:     authorizer,
		Log:      log,
		Config:   cf,
	})

	authMiddleware := auth.NewMiddleware(env.Auth, log)

	env.Handle("/users/login", users.LoginPOSTHandler).Methods("POST")
	env.Handle("/users/signup", users.SignupPOSTHandler).Methods("POST")
	env.Handle("/users/details", users.DetailsHandler).WithRole(users.UserRole).Methods("GET")
	env.Handle("/users/edit", users.EditHandler).WithRole(users.UserRole).Methods("POST")

	env.Handle("/movies/polochon", movies.FromPolochon).WithRole(users.UserRole).Methods("GET")
	env.Handle("/movies/{id:tt[0-9]+}/refresh", movies.GetDetailsHandler).WithRole(users.UserRole).Methods("POST")
	env.Handle("/movies/{id:tt[0-9]+}", movies.DeleteHandler).WithRole(users.AdminRole).Methods("DELETE")
	env.Handle("/movies/explore", extmedias.Explore).WithRole(users.UserRole).Methods("GET")
	env.Handle("/movies/refresh", extmedias.Refresh).WithRole(users.UserRole).Methods("POST")
	env.Handle("/movies/search", movies.SearchMovie).WithRole(users.UserRole).Methods("POST")

	env.Handle("/shows/polochon", shows.FromPolochon).WithRole(users.UserRole).Methods("GET")
	env.Handle("/shows/{id:tt[0-9]+}", shows.GetDetailsHandler).WithRole(users.UserRole).Methods("GET")
	env.Handle("/shows/{id:tt[0-9]+}/seasons/{season:[0-9]+}/episodes/{episode:[0-9]+}", shows.RefreshEpisodeHandler).WithRole(users.UserRole).Methods("POST")
	env.Handle("/shows/{id:tt[0-9]+}/refresh", shows.RefreshDetailsHandler).WithRole(users.UserRole).Methods("POST")
	env.Handle("/shows/refresh", extmedias.RefreshShows).WithRole(users.UserRole).Methods("POST")
	env.Handle("/shows/explore", extmedias.ExploreShows).WithRole(users.UserRole).Methods("GET")
	env.Handle("/shows/search", shows.SearchShow).WithRole(users.UserRole).Methods("POST")

	env.Handle("/torrents", torrents.DownloadHandler).WithRole(users.UserRole).Methods("POST")

	env.Handle("/wishlist/shows", shows.GetWishlistHandler).WithRole(users.UserRole).Methods("GET")
	env.Handle("/wishlist/shows/{id:tt[0-9]+}", shows.AddToWishlist).WithRole(users.UserRole).Methods("POST")
	env.Handle("/wishlist/shows/{id:tt[0-9]+}", shows.DeleteFromWishlist).WithRole(users.UserRole).Methods("DELETE")

	env.Handle("/wishlist/movies", movies.GetWishlistHandler).WithRole(users.UserRole).Methods("GET")
	env.Handle("/wishlist/movies/{id:tt[0-9]+}", movies.AddToWishlist).WithRole(users.UserRole).Methods("POST")
	env.Handle("/wishlist/movies/{id:tt[0-9]+}", movies.DeleteFromWishlist).WithRole(users.UserRole).Methods("DELETE")

	n := negroni.Classic()
	n.Use(authMiddleware)
	n.Use(negroni.NewStatic(http.Dir(cf.PublicDir)))
	n.Use(gzip.Gzip(gzip.DefaultCompression))
	n.UseHandler(env.Router)
	n.Run(":" + cf.Port)
}
